/*	imhelper - Text Input Method Helper
 *
 *  ptimhelper.c - 
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com>,
 *           LUO Lei <Srella@163.com>
 *
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <X11/Xatom.h>
#include <stdio.h>

#include "standard_im.h"
#include "sim_helper_gtk.h"

#include "ptimhelperTSeng.h"
#include "ptimhelperfcitx.h"

//#define IMH_TWOSTEP_INDEX		0
//#define IMH_ONESTEP_INDEX		1
#define IMH_TS_INDEX					0
#define IMH_FCITX_INDEX			1

static GtkWidget * helper_window = NULL;
static int tIMindex;

//extern int candidatesnumberEN;

/* global IM Manger call-back Functinos */
IMM_Functions * imm_functions = NULL;

static const IME_Helper_Info imh_info [] =
{
	{
	 	"sim-ts-helper-gtk",
		"en_US",
		"TouchScreen Input Helper",
		"For TS engine",
		NULL,
		0x0 | CAP_TS_INPUT,
	},

	{
	 	"sim-ts-helper-gtk",
		"cn_ZH",
		"TouchScreen Input Helper Fcitx",
		"For fcitx engine",
		NULL,
		0x0 | CAP_TS_INPUT,
	}
};

int 
imh_gtk_get_number_of_helper (void)
{
	return (sizeof(imh_info) / sizeof(IME_Helper_Info)); 
}

int 
imh_gtk_get_helper_info (int index, IME_Helper_Info * info)
{
	if (index < 0 || index > imh_gtk_get_number_of_helper() - 1)
	{
		return 0;
	}

#ifdef	DEBUG
	assert(info);
#endif
	memcpy(info, &(imh_info[index]), sizeof(IME_Helper_Info)); 
	
	return 1;
}

void 
imh_gtk_initialize (const char * name)
{
	int i;
	int helpercount = imh_gtk_get_number_of_helper();

	for(i = 0; i<= helpercount; i++)
	{
		if(strcmp(name, imh_info[i].name) == 0) break;
	}

	if(i >= helpercount) 
	{
		g_print("Didn't found the helper in configure file!\n");	
		return;
	}
	else					/*find corresponding helper*/
	{
		if(tIMindex == IMH_TS_INDEX) 
		{
			ptim_helper_TS_eng_finalize ();
		}
		else if(tIMindex == IMH_FCITX_INDEX) 
		{
			ptim_helper_fcitx_finalize ();
		}

		if (helper_window)
		{
			g_print("%s(): helper_window(destroy) = 0x%08x\n", __FUNCTION__, (guint32)helper_window);
	
			gtk_widget_destroy(helper_window);
			helper_window = NULL;
		}

		tIMindex = i;
		if(tIMindex == IMH_TS_INDEX) 
		{
     helper_window = ptim_helper_TS_eng_new(); 
		}
		else if(tIMindex == IMH_FCITX_INDEX) 
		{
     helper_window = ptim_helper_fcitx_new(); 
		}
		else return;	
	}
	g_print("%s(): helper_window(new) = 0x%08x\n", __FUNCTION__, (guint32)helper_window);

	return;
}

void 
imh_gtk_finalize (void)
{
	ptim_helper_TS_eng_finalize ();
	ptim_helper_fcitx_finalize();
	if (helper_window)
	{
		g_print("%s(): helper_window(destroy) = 0x%08x\n", __FUNCTION__, (guint32)helper_window);
	
		gtk_widget_destroy(helper_window);
		helper_window = NULL;
	}
	
	return;
}

void 
imh_gtk_show_preedit_candidates_list (const IME_CandidatesList * list)
{
	gtk_widget_show_all(helper_window);

	return;
}

void 
imh_gtk_show_candidates_list (const IME_CandidatesList * list)
{
	GString * str;
	int i;

	if(tIMindex == IMH_FCITX_INDEX)
	{
		if (list != NULL)
		{
			str = g_string_new("");
		
			for (i = 0; i < 5; i++)
			{
					g_string_append(str, "<span foreground = \"black\" background = \"white\">");
					
					g_string_append(str, "<span foreground = \"blue\" background = \"white\" rise = \"5000\"  size = \"xx-small\">");


					g_string_append_printf(str, "%-2d", i + 1);

					g_string_append(str, "</span>");
					g_string_append(str, ((char**)list)[i]);
					g_string_append(str, "</span> ");
				
			}

			ptim_helper_fcitx_set_cands(PTIM_HELPER_FCITX(helper_window), str->str);
			g_string_free(str, TRUE); 
		}
		else
		{
			ptim_helper_fcitx_set_cands(PTIM_HELPER_FCITX(helper_window), "");
		}
	}


	gtk_widget_show_all(helper_window);

	return;
}

void 
imh_gtk_hide_all_candidates_list (void)
{
	if (helper_window)
	{
		gtk_widget_hide_all(helper_window);
	}
	
	return;
}

void 
imh_gtk_notify_ime_status (IME_Status status)
{
//	ime_status = status;
	return;
}

void 
imh_gtk_notify_numeric_display (IME_Bool numeric_display)
{
//	imh_numeric_display = numeric_display;
	return;
}

void 
imh_gtk_notify_client_size (int client_x, int client_y, int client_width, int client_height)
{
	int x, y;
	int width, height;

	gtk_window_get_size(GTK_WINDOW(helper_window), &width, &height);

	x = client_x;
	y = client_y + client_height - height;

	gtk_window_move(GTK_WINDOW(helper_window), x, y);
	gtk_window_resize(GTK_WINDOW(helper_window), client_width, height);

#ifdef	DEBUG
 	fprintf(stdout, "helper: [%3d, %3d], [%3d, %3d]\n", x, y, client_width, height);
#endif
	return;
}

IME_Bool 
imh_register_module (const IMM_Functions * imm, IMH_Functions * imh)
{
#ifdef	DEBUG
	assert(imm);
	assert(imh);
#endif

	/* set global IMM fucntions pointer */
	imm_functions = imm;

	imh->initialize												= imh_gtk_initialize;
	imh->finalize													= imh_gtk_finalize;
	imh->show_preedit_candidates_list	= imh_gtk_show_preedit_candidates_list;
	imh->show_candidates_list						= imh_gtk_show_candidates_list;
	imh->hide_all_candidates_list			= imh_gtk_hide_all_candidates_list;
	imh->notify_ime_status								= imh_gtk_notify_ime_status;
	imh->notify_numeric_display					= imh_gtk_notify_numeric_display;
	imh->notify_client_size							= imh_gtk_notify_client_size;
	imh->get_number_of_helper						= imh_gtk_get_number_of_helper;
	imh->get_helper_info									= imh_gtk_get_helper_info;

	return IME_TRUE;
}

gboolean 
helper_module_init (GTypeModule * type_module)
{
 	ptim_helper_TS_eng_register_type(type_module);
 	ptim_helper_fcitx_register_type(type_module);
	return TRUE;
}
