/*	imhelper - Text Input Method Helper
 *
 *  ptimhelperTSeng.c - 
 *
 *	Authors: YE Nan <nan.ye@orange-ftgourp.com>,
 *           LUO Lei <Srella@163.com>
 *
 *	
 *	This software and associated documentation files (the "Software") 
 *	are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *	All Rights Reserved. 
 *
 *	A copyright license is hereby granted for redistribution and use of 
 *	the Software in source and binary forms, with or without modification, 
 *	provided that the following conditions are met: 
 *	- Redistributions of source code must retain the above copyright notice, 
 *	this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 * 	notice, this copyright license and the following disclaimer in the 
 *	documentation and/or other materials provided with the distribution. 
 *	- Neither the name of LiPS  nor the names of its Members may be used 
 *	to endorse or promote products derived from the Software without 
 *	specific prior written permission. 
 *
 *	A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *	to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *	distribute any implementation compliant with the any specification adopted 
 *	by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *	and non-discriminatory terms and conditions and under reciprocity, as 
 *	regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *	THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *	"AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *	THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *	AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *	ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *	SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *	PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *	OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *	WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *	POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdlib.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <gdk/gdkx.h>

#include "ptimhelperTSeng.h"
#include "standard_im.h"

static void	ptim_helper_TS_eng_class_init	(PtimHelperTSengClass		*klass);
static void	ptim_helper_TS_eng_init					(PtimHelperTSeng				*helper);

GType ptim_helper_TS_eng_type = 0;
PtimHelperTSeng * global_helper;

gboolean cap_status;
gboolean shift_status;
gboolean symbol_status;

extern IMM_Functions * imm_functions;

void
ptim_helper_TS_eng_register_type (GTypeModule * type_module)
{
  if (!ptim_helper_TS_eng_type)
	{
		static const GTypeInfo helper_TS_eng_info =
		{
			sizeof (PtimHelperTSengClass),
			NULL,		/* base_init */
			NULL,		/* base_finalize */
			(GClassInitFunc) ptim_helper_TS_eng_class_init,
			NULL,		/* class_finalize */
			NULL,		/* class_data */
			sizeof (PtimHelperTSeng),
			0,			/* n_preallocs */
			(GInstanceInitFunc) ptim_helper_TS_eng_init,
		};

		ptim_helper_TS_eng_type = g_type_module_register_type(type_module,
																													 GTK_TYPE_WINDOW,
																													 "PtimHelperTSeng", 
																													 &helper_TS_eng_info, 
																													 0);
	}
	return;
}

static void
ptim_helper_TS_eng_class_init (PtimHelperTSengClass * klass)
{
  g_print("TS_eng init\n");
	return;
}

static gboolean
ptim_helper_TS_eng_expose_cb (GtkWidget * widget, GdkEventExpose * event, gpointer user_data)
{
	gchar * res_path;
	res_path = getenv("PTIM_PIXMAPS_PATH");

	gchar * real_path = NULL;
	gchar * cap_path = NULL;		

	GdkPixbuf * pbuf_keyboard = NULL;
	GdkPixbuf * pbuf_keyboardone = NULL;

	GError * error = NULL;

	if (!res_path)
	{
		return FALSE;
	}

	/*draw symbol keyboard*/
	if(symbol_status == TRUE)	
	{
   	real_path = g_build_filename(res_path, "keyboard_symbols.png", NULL);
		pbuf_keyboard = gdk_pixbuf_new_from_file_at_size(real_path, 240, 79, &error);

		if (error != NULL)
		{
			g_print("%s(): symbol keyboard load error: %s\n", __FUNCTION__, error->message);
			g_error_free(error);
			error = NULL;
		}
		g_free(real_path);
		
		if (pbuf_keyboard != NULL)
		{
			gdk_draw_pixbuf(widget->window, widget->style->white_gc, pbuf_keyboard, 
																			0, 0, 0, 0, -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
		}
		return FALSE;
	}

	/*draw keyboard*/
	if(cap_status == shift_status)
	{
   	real_path = g_build_filename(res_path, "keyboard_lowercase.png", NULL);
	}
	else
	{
   	real_path = g_build_filename(res_path, "keyboard_uppercase.png", NULL);
	}

	pbuf_keyboard = gdk_pixbuf_new_from_file_at_size(real_path, 240, 79, &error);

	if (error != NULL)
	{
		g_print("%s(): keyboard load error: %s\n", __FUNCTION__, error->message);
		g_error_free(error);
		error = NULL;
	}
	g_free(real_path);
			
	if (pbuf_keyboard != NULL)
	{
		gdk_draw_pixbuf(widget->window, widget->style->white_gc, pbuf_keyboard, 
																		0, 0, 0, 0, -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
	}

	/*draw cap key*/
	if(cap_status == TRUE)
	{
   	cap_path = g_build_filename(res_path, "keyboard_capson.png", NULL);
		pbuf_keyboardone = gdk_pixbuf_new_from_file_at_size(cap_path, 43, 15, &error);

		if (error != NULL)
		{
			g_print("%s(): keyboardone load error: %s\n", __FUNCTION__, error->message);
			g_error_free(error);
			error = NULL;
		}
		g_free(cap_path);
		
		if (pbuf_keyboardone != NULL)
		{
			gdk_draw_pixbuf(widget->window, widget->style->white_gc, pbuf_keyboardone,
																			 0, 0, 4, 46, -1, -1, GDK_RGB_DITHER_NONE, 0, 0);
		}
	}

	return FALSE;
}

static char *
unicode2utf8(const gunichar2 charval)
{
	GError * error = NULL;
	gunichar2 tmp = charval;
	
	char * utf8char = g_utf16_to_utf8(&tmp, 1, NULL, NULL, &error);
	g_error_free(error);

	return utf8char;
}


static void 
helper_btn_handler(GtkWidget *w, GtkWidget *w_backup)
{
	char *str = (char*) malloc (2 * sizeof(char));
	str[1] = '\0';

	gboolean systemkey = FALSE;
	guint systemkeyval;

	int x = 0, y = 0;
	gtk_widget_get_pointer(w, &x, &y);

	if(symbol_status == TRUE)			/*symbol keyboard*/
	{
		if(y >= 0 && y <= 18)					/*the first line*/
		{
			if(x >= 0 && x <= 25) str[0] = '(';
			else if(x >= 26 && x <= 47) str[0] = ')';
			else if(x >= 48 && x <= 68) str[0] = '[';
			else if(x >= 69 && x <= 89) str[0] = ']';
			else if(x >= 90 && x <= 110) str[0] = '{';
			else if(x >= 111 && x <= 131) str[0] = '}';
			else if(x >= 132 && x <= 152) str[0] = '<';
			else if(x >= 153 && x <= 173) str[0] = '>';
			else if(x >= 174 && x <= 194) str[0] = '"';
			else if(x >= 195 && x <= 215) str[0] = '\'';
			else if(x >= 216 && x <= 240) 		/*backspace*/
			{
				systemkey = TRUE; 
				systemkeyval = GDK_BackSpace;
			}
			else return;
		}
		else if(y >= 19 && y <= 33)		/*the second line*/
		{	
			if(x >= 0 && x <= 25) str[0] = '?';
			else if(x >= 26 && x <= 47) str[0] = '!';
			else if(x >= 48 && x <= 68) str[0] = '@';
			else if(x >= 69 && x <= 89) str[0] = '#';
			else if(x >= 90 && x <= 110) str[0] = '$';
			else if(x >= 111 && x <= 131) str[0] = '%';
			else if(x >= 132 && x <= 152) str[0] = '^';
			else if(x >= 153 && x <= 173) str[0] = '&';
			else if(x >= 174 && x <= 194) str[0] = '*';
			else if(x >= 195 && x <= 215) str[0] = '-';
			else if(x >= 216 && x <= 240) str[0] = '+';
			else return;
		}
		else if(y >= 34 && y <= 47)		/*the third line*/
		{
			if(x >= 0 && x <= 25) str[0] = str[0] = '~';
			else if(x >= 26 && x <= 47) str[0] = '`';
			else if(x >= 48 && x <= 68) {g_free(str); str = unicode2utf8(0x2122);}
			else if(x >= 69 && x <= 89) {g_free(str); str = unicode2utf8(0x00BA);}
			else if(x >= 90 && x <= 110) {g_free(str); str = unicode2utf8(0x2022);}
			else if(x >= 111 && x <= 131) {g_free(str); str = unicode2utf8(0x00B1);}
			else if(x >= 132 && x <= 152) str[0] = '|';
			else if(x >= 153 && x <= 173) str[0] = '\\';
			else if(x >= 174 && x <= 194) str[0] = '/';
			else if(x >= 195 && x <= 215) str[0] = '_';
			else if(x >= 216 && x <= 240) str[0] = '=';
			else return;
		}
		else if(y >= 48 && y <= 61)		/*the fourth line*/
		{
			if(x >= 0 && x <= 47) 		/*uppercase/lowercase switch*/
			{
				g_free(str);
				return;
			}
			else if(x >= 48 && x <= 68) {g_free(str); str = unicode2utf8(0x00A9);}
			else if(x >= 69 && x <= 89) {g_free(str); str = unicode2utf8(0x00AE);}
			else if(x >= 90 && x <= 110) {g_free(str); str = unicode2utf8(0x00A5);}
			else if(x >= 111 && x <= 131) {g_free(str); str = unicode2utf8(0x00A3);}
			else if(x >= 132 && x <= 152) {g_free(str); str = unicode2utf8(0x00A2);}
			else if(x >= 153 && x <= 173) {g_free(str); str = unicode2utf8(0x20AC);}
			else if(x >= 174 && x <= 194) {g_free(str); str = unicode2utf8(0x2030);}
			else if(x >= 195 && x <= 240)		/*enter*/
			{
				systemkey = TRUE; 
				systemkeyval = GDK_Return;
			}
			else return;
		}
		else if(y >= 62 && y <= 80)		/*the fiveth line*/
		{
			if(x >= 0 && x <= 47)
			{
				g_free(str);
				return;
			}
			else if(x >= 48 && x <= 68) str[0] = ',';
			else if(x >= 69 && x <= 89) str[0] = '.';
			else if(x >= 90 && x <= 152) str[0] = ' ';
			else if(x >= 153 && x <= 173) str[0] = ':';
			else if(x >= 174 && x <= 194) str[0] = ';';
			else if(x >= 195 && x <= 240)
			{
				symbol_status = !symbol_status;
				ptim_helper_TS_eng_expose_cb(global_helper->keyboard, NULL, NULL); 
				g_free(str);
				return;
			}
			else return;
		}
		else return;
	}

	else if(symbol_status == FALSE)		/*normal keyboard*/
	{
		if(y >= 0 && y <= 18)					/*the first line*/
		{
			if(x >= 0 && x <= 25) str[0] = (cap_status == shift_status) ? '1' : '!';
			else if(x >= 26 && x <= 47) str[0] = (cap_status == shift_status) ? '2' : '@';
			else if(x >= 48 && x <= 68) str[0] = (cap_status == shift_status) ? '3' : '#';
			else if(x >= 69 && x <= 89) str[0] = (cap_status == shift_status) ? '4' : '$';
			else if(x >= 90 && x <= 110) str[0] = (cap_status == shift_status) ? '5' : '%';
			else if(x >= 111 && x <= 131) str[0] = (cap_status == shift_status) ? '6' : '^';
			else if(x >= 132 && x <= 152) str[0] = (cap_status == shift_status) ? '7' : '&';
			else if(x >= 153 && x <= 173) str[0] = (cap_status == shift_status) ? '8' : '*';
			else if(x >= 174 && x <= 194) str[0] = (cap_status == shift_status) ? '9' : '(';
			else if(x >= 195 && x <= 215) str[0] = (cap_status == shift_status) ? '0' : ')';
			else if(x >= 216 && x <= 240) 		/*backspace*/
			{
				systemkey = TRUE; 
				systemkeyval = GDK_BackSpace;
			}
			else return;
		}
		else if(y >= 19 && y <= 33)		/*the second line*/
		{	
			if(x >= 0 && x <= 25) str[0] = (cap_status == shift_status) ? 'q' : 'Q';
			else if(x >= 26 && x <= 47) str[0] = (cap_status == shift_status) ? 'w' : 'W';
			else if(x >= 48 && x <= 68) str[0] = (cap_status == shift_status) ? 'e' : 'E';
			else if(x >= 69 && x <= 89) str[0] = (cap_status == shift_status) ? 'r' : 'R';
			else if(x >= 90 && x <= 110) str[0] = (cap_status == shift_status) ? 't' : 'T';
			else if(x >= 111 && x <= 131) str[0] = (cap_status == shift_status) ? 'y' : 'Y';
			else if(x >= 132 && x <= 152) str[0] = (cap_status == shift_status) ? 'u' : 'U';
			else if(x >= 153 && x <= 173) str[0] = (cap_status == shift_status) ? 'i' : 'I';
			else if(x >= 174 && x <= 194) str[0] = (cap_status == shift_status) ? 'o' : 'O';
			else if(x >= 195 && x <= 215) str[0] = (cap_status == shift_status) ? 'p' : 'P';
			else if(x >= 216 && x <= 240) str[0] = (cap_status == shift_status) ? '\\' : '|';
			else return;
		}
		else if(y >= 34 && y <= 47)		/*the third line*/
		{
			if(x >= 0 && x <= 25) str[0] = (cap_status == shift_status) ? 'a' : 'A';
			else if(x >= 26 && x <= 47) str[0] = (cap_status == shift_status) ? 's' : 'S';
			else if(x >= 48 && x <= 68) str[0] = (cap_status == shift_status) ? 'd' : 'D';
			else if(x >= 69 && x <= 89) str[0] = (cap_status == shift_status) ? 'f' : 'F';
			else if(x >= 90 && x <= 110) str[0] = (cap_status == shift_status) ? 'g' : 'G';
			else if(x >= 111 && x <= 131) str[0] = (cap_status == shift_status) ? 'h' : 'H';
			else if(x >= 132 && x <= 152) str[0] = (cap_status == shift_status) ? 'j' : 'J';
			else if(x >= 153 && x <= 173) str[0] = (cap_status == shift_status) ? 'k' : 'K';
			else if(x >= 174 && x <= 194) str[0] = (cap_status == shift_status) ? 'l' : 'L';
			else if(x >= 195 && x <= 215) str[0] = (cap_status == shift_status) ? ';' : ':';
			else if(x >= 216 && x <= 240) str[0] = (cap_status == shift_status) ? '\'' : '"';
			else return;
		}
		else if(y >= 48 && y <= 61)		/*the fourth line*/
		{
			if(x >= 0 && x <= 47) 		/*uppercase/lowercase switch*/
			{
				cap_status = !cap_status;
				ptim_helper_TS_eng_expose_cb(global_helper->keyboard, NULL, NULL); 
				g_free(str);
				return;
			}
			else if(x >= 48 && x <= 68) str[0] = (cap_status == shift_status) ? 'z' : 'Z';
			else if(x >= 69 && x <= 89) str[0] = (cap_status == shift_status) ? 'x' : 'X';
			else if(x >= 90 && x <= 110) str[0] = (cap_status == shift_status) ? 'c' : 'C';
			else if(x >= 111 && x <= 131) str[0] = (cap_status == shift_status) ? 'v' : 'V';
			else if(x >= 132 && x <= 152) str[0] = (cap_status == shift_status) ? 'b' : 'B';
			else if(x >= 153 && x <= 173) str[0] = (cap_status == shift_status) ? 'n' : 'N';
			else if(x >= 174 && x <= 194) str[0] = (cap_status == shift_status) ? 'm' : 'M';
			else if(x >= 195 && x <= 240)		/*enter*/
			{
				systemkey = TRUE; 
				systemkeyval = GDK_Return;
			}
			else return;
		}
		else if(y >= 62 && y <= 80)		/*the fiveth line*/
		{
			if(x >= 0 && x <= 47)
			{
				shift_status = !shift_status;
				ptim_helper_TS_eng_expose_cb(global_helper->keyboard, NULL, NULL); 
				g_free(str);
				return;
			}
			else if(x >= 48 && x <= 68) str[0] = (cap_status == shift_status) ? '/' : '?';
			else if(x >= 69 && x <= 89) str[0] = (cap_status == shift_status) ? '-' : '_';
			else if(x >= 90 && x <= 152) str[0] = ' ';
			else if(x >= 153 && x <= 173) str[0] = (cap_status == shift_status) ? ',' : '<';
			else if(x >= 174 && x <= 194) str[0] = (cap_status == shift_status) ? '.' : '>';
			else if(x >= 195 && x <= 240)
			{
				symbol_status = !symbol_status;
				shift_status = FALSE;
				ptim_helper_TS_eng_expose_cb(global_helper->keyboard, NULL, NULL); 
				g_free(str);
				return;
			}
			else return;
		}
		else return;
	}

	if(systemkey)
	{
		Window toplevel_window;
		int revert = RevertToNone;
		XKeyEvent event;
    
		XGetInputFocus(GDK_DISPLAY(), &toplevel_window, &revert);   

		event.display     = GDK_DISPLAY();
		event.window      = toplevel_window;
		event.root        = GDK_ROOT_WINDOW();
		event.subwindow   = None;
		event.state       = 0;
		event.x           = 1;
		event.y           = 1;
		event.x_root      = 1;
		event.y_root      = 1;
 		event.same_screen = True;
		event.keycode     = XKeysymToKeycode(GDK_DISPLAY(), systemkeyval);
		event.time        = CurrentTime; 
		event.type        = KeyPress;

	  XSendEvent(event.display, event.window, TRUE, KeyPressMask, (XEvent*)&event);

		g_free(str);
		return;
	}

	imm_functions->commit_string(str);
	g_free(str);	

	if(shift_status == TRUE)
	{
		shift_status = FALSE;
		ptim_helper_TS_eng_expose_cb(global_helper->keyboard, NULL, NULL); 
	}
	return;
}

static void	
ptim_helper_TS_eng_init (PtimHelperTSeng * helper)
{
	global_helper = helper;
	cap_status = FALSE;
	shift_status = FALSE;
	symbol_status = FALSE;

	GtkWidget *eventbox;

	helper->keyboard = gtk_drawing_area_new();
	gtk_widget_set_size_request (helper->keyboard, 240, 79);
	g_signal_connect(G_OBJECT(helper->keyboard), "expose-event", G_CALLBACK(ptim_helper_TS_eng_expose_cb), NULL);

	eventbox = gtk_event_box_new();

	gtk_container_add(GTK_CONTAINER(helper), eventbox);
	gtk_container_add(GTK_CONTAINER(eventbox), helper->keyboard);

  gtk_widget_set_events (eventbox, GDK_BUTTON_PRESS_MASK);
  g_signal_connect (G_OBJECT (eventbox), "button_press_event",
	             			 G_CALLBACK (helper_btn_handler), eventbox);

	if (TRUE)
	{
		GdkColor color_w;
		gdk_color_parse("white", &color_w);
		gtk_widget_modify_bg(GTK_WIDGET(helper), GTK_STATE_NORMAL, &color_w);		
	}

	gtk_window_move(GTK_WINDOW(helper), 0, gdk_screen_height() / 2);
	gtk_widget_set_size_request(GTK_WIDGET(helper), gdk_screen_width(), -1);

	return;
}

GtkWidget *
ptim_helper_TS_eng_new (void)
{
  return g_object_new(PTIM_TYPE_HELPER_TS_ENG, 
											"type", GTK_WINDOW_TOPLEVEL,
											"type-hint", GDK_WINDOW_TYPE_HINT_DOCK,
											"accept-focus", FALSE,
											NULL);
}

void		
ptim_helper_TS_eng_finalize (void)
{
	return;
}


void
ptim_helper_TS_eng_set_cands (PtimHelperTSeng * helper, const gchar * cand_str)
{
	return;
}
